#!/usr/bin/env python3
"""
python MSISE00.py -q -w tests/ref4.nc -a 200 -gs 90 90 -t 2017-03-01T12 2017-03-01T14
"""
from pathlib import Path
import subprocess
import sys
import pytest
import xarray
import xarray.tests
import msise00
import msise00.worldgrid

R = Path(__file__).resolve().parent
time = ["2017-03-01T12", "2017-03-01T13"]
altkm = 200.0
script = R.parent / "MSISE00.py"


def test_multiple_time():
    pytest.importorskip("netCDF4")
    ref = xarray.open_dataset(R / "ref4.nc")

    lat, lon = msise00.worldgrid.latlonworldgrid(90, 90)

    try:
        dat_mod = msise00.run(time, altkm, lat, lon).squeeze()
    except ConnectionError:
        pytest.skip("unable to download RecentIndices.txt")
    xarray.tests.assert_allclose(ref, dat_mod)


@pytest.mark.skipif(not script.is_file(), reason="demo script not available")
def test_script(tmp_path):
    pytest.importorskip("netCDF4")
    ref = xarray.open_dataset(R / "ref4.nc")

    fn = tmp_path / "test.nc"
    cmd = [sys.executable, str(script), "-q", "-w", str(fn), "-gs", "90", "90", "-a", str(altkm), "-t"] + time
    print(" ".join(cmd))
    subprocess.check_call(cmd, cwd=R.parent)

    dat = xarray.open_dataset(fn)
    xarray.tests.assert_allclose(ref, dat)


if __name__ == "__main__":
    pytest.main(["-v", __file__])



#!/usr/bin/env python
from datetime import datetime
import numpy as np
import pytest
from pytest import approx
import msise00
from pathlib import Path

R = Path(__file__).parent
reffn = R / "ccmc.log"


def test_ccmc():
    t = datetime(2001, 2, 2, 8, 0, 0)
    glat = 60.0
    glon = -70.0
    altkm = 400.0
    indices = {"f107s": 163.6666, "f107": 146.7, "Ap": 7}

    A = np.loadtxt(reffn, skiprows=25)

    atmos = msise00.run(t, altkm, glat, glon, indices)
    assert atmos.f107s == approx(163.6666)
    assert atmos.f107 == approx(146.7)
    assert atmos.Ap == approx(7)
    assert A[0] == approx(altkm)
    assert A[1] == approx(atmos["O"].item() / 1e6, rel=0.05)
    assert A[2] == approx(atmos["N2"].item() / 1e6, rel=0.3)
    assert A[3] == approx(atmos["O2"].item() / 1e6, rel=0.35)
    assert A[7] == approx(atmos["He"].item() / 1e6, rel=0.2)
    assert A[8] == approx(atmos["Ar"].item() / 1e6, rel=0.6)
    assert A[9] == approx(atmos["H"].item() / 1e6, rel=0.1)
    assert A[10] == approx(atmos["N"].item() / 1e6, rel=0.3)
    assert A[11] == approx(atmos["AnomalousO"].item() / 1e6, rel=0.3)

    assert A[5] == approx(atmos["Tn"].item(), rel=0.1)
    assert A[6] == approx(atmos["Texo"].item(), rel=0.1)


def test_past():
    t = datetime(2013, 3, 31, 12)
    altkm = 150.0
    glat = 65.0
    glon = -148.0

    try:
        atmos = msise00.run(t, altkm, glat, glon)
    except ConnectionError:
        pytest.xfail("unable to download RecentIndices.txt")

    assert atmos["He"].ndim == 4
    assert atmos["He"].size == 1
    dims = list(atmos.dims)
    assert ["alt_km", "lat", "lon", "time"] == dims

    try:  # daily resolution
        assert atmos["He"].item() == approx(4365665440000.0)
        assert atmos["O"].item() == approx(9612878760000000.0)
        assert atmos["N2"].item() == approx(3.15052301e16)
        assert atmos["O2"].item() == approx(2312921490000000.0)
        assert atmos["Ar"].item() == approx(71974164400000.0)
        assert atmos["Total"].item() == approx(1.84757176e-09)
        assert atmos["N"].item() == approx(9424203680000.0)
        assert atmos["AnomalousO"].item() == approx(1.17317104e-14)

        assert atmos["Tn"].item() == approx(682.538, abs=0.01)
        assert atmos["Texo"].item() == approx(948.350, abs=0.01)
    except AssertionError:  # monthly resolution
        assert atmos["He"].item() == approx(4200180480000.0)
        assert atmos["O"].item() == approx(9338048100000000.0)
        assert atmos["N2"].item() == approx(3.23984781e16)
        assert atmos["O2"].item() == approx(2413811350000000.0)
        assert atmos["Ar"].item() == approx(81071685200000.0)
        assert atmos["Total"].item() == approx(1.88774951e-09)
        assert atmos["N"].item() == approx(9310465690000.0)
        assert atmos["AnomalousO"].item() == approx(5.3806201e-15)

        assert atmos["Tn"].item() == approx(699.021, abs=0.01)
        assert atmos["Texo"].item() == approx(1000.513, abs=0.01)

    assert atmos.species == ["He", "O", "N2", "O2", "Ar", "Total", "H", "N", "AnomalousO"]


def test_forecast():
    t = datetime(2023, 3, 31, 12)
    altkm = 150.0
    glat = 65.0
    glon = -148.0

    atmos = msise00.run(t, altkm, glat, glon)

    assert atmos["He"].ndim == 4
    assert atmos["He"].size == 1
    dims = list(atmos.dims)
    assert ["alt_km", "lat", "lon", "time"] == dims

    assert atmos["He"].item() == approx(1240641440000.0)
    assert atmos["O"].item() == approx(3047968100000000.0)
    assert atmos["N2"].item() == approx(3.597249e16)
    assert atmos["O2"].item() == approx(7343665540000000.0)
    assert atmos["Ar"].item() == approx(334703983000000.0)
    assert atmos["Total"].item() == approx(2.16534546e-09)
    assert atmos["N"].item() == approx(2576280450000.0)
    assert atmos["AnomalousO"].item() == approx(8.75042368e-16)

    assert atmos.species == ["He", "O", "N2", "O2", "Ar", "Total", "H", "N", "AnomalousO"]

    assert atmos["Tn"].item() == approx(671.513672)
    assert atmos["Texo"].item() == approx(883.342529)


if __name__ == "__main__":
    pytest.main(["-v", __file__])



#!/usr/bin/env python3
"""
Regenerate test data:

./MSISE00.py -q -w tests/ref6.nc -a 200 -t 2017-03-01T12 -c 65 -148
./MSISE00.py -q -w tests/ref5.nc -a 100 -t 2017-03-01T12 -c 65 -148
"""
from pathlib import Path
import subprocess
import sys
import pytest
import xarray
import xarray.tests
import msise00
import msise00.worldgrid

R = Path(__file__).resolve().parent
lat = 65
lon = -148
time = "2017-03-01T12"
script = R.parent / "MSISE00.py"


@pytest.mark.parametrize("altkm,reffn", [(100.0, "ref5.nc"), (200.0, "ref6.nc")])
def test_one_loc_one_time(altkm, reffn):
    pytest.importorskip("netCDF4")
    ref = xarray.open_dataset(R / reffn)

    try:
        dat_mod = msise00.run(time, altkm, lat, lon).squeeze()
    except ConnectionError:
        pytest.skip("unable to download RecentIndices.txt")
    xarray.tests.assert_allclose(ref, dat_mod)


@pytest.mark.skipif(not script.is_file(), reason="demo script not available")
@pytest.mark.parametrize("altkm,reffn", [(100.0, "ref5.nc"), (200.0, "ref6.nc")])
def test_script(altkm, reffn, tmp_path):
    pytest.importorskip("netCDF4")
    ref = xarray.open_dataset(R / reffn)

    fn = tmp_path / "test.nc"
    cmd = [sys.executable, str(script), "-q", "-w", str(fn), "-a", str(altkm), "-t", time, "-c", str(lat), str(lon)]
    print(" ".join(cmd))
    subprocess.check_call(cmd)

    dat = xarray.open_dataset(fn)
    xarray.tests.assert_allclose(ref, dat)


if __name__ == "__main__":
    pytest.main(["-v", __file__])



#!/usr/bin/env python
"""
Time profile example
"""
import numpy as np
from datetime import datetime, timedelta
from matplotlib.pyplot import show

import msise00.plots as msplots
import msise00

glat = 65.1
glon = -147.5
alt_km = 200
t0 = datetime(2015, 12, 13, 10, 0, 0)
t1 = datetime(2015, 12, 14, 10, 0, 0)

times = np.arange(t0, t1, timedelta(hours=1))

atmos = msise00.run(times, alt_km, glat, glon)

msplots.plotgtd(atmos)

show()



#!/usr/bin/env python3
import subprocess
import shutil
from pathlib import Path
from typing import List

R = Path(__file__).parent
RS = R / "fortran"

SOURCES = list(map(str, [RS / "nrlmsise00_sub.for", RS / "msise00_driver.f90"]))
EXE = R / "msise00_driver"
EXE_OPT = ["-o", str(EXE)]


def build(sources: List[str] = SOURCES, compiler: str = "gfortran", exe_opt: List[str] = EXE_OPT) -> str:
    """
    Attempt to compile code instead of using setup.py
    """
    if isinstance(sources, (str, Path)):
        sources = [sources]

    fc = shutil.which(compiler)
    if not fc:
        raise FileNotFoundError(fc)

    for s in SOURCES:
        if not Path(s).is_file():
            raise FileNotFoundError(s)

    sources = list(map(str, sources))

    subprocess.check_call([str(fc)] + sources + EXE_OPT)

    return shutil.which(EXE.name, path=str(R))


if __name__ == "__main__":
    exe = build()
    print("compiled", exe)



#!/usr/bin/env python
"""
Example:
Poker Flat Research Range altitude profile
"""
import numpy as np
from datetime import datetime
from matplotlib.pyplot import show

import msise00.plots as msplots
import msise00

glat = 65.1
glon = -147.5
alt_km = np.arange(50, 1000, 20.0)
time = datetime(2015, 12, 13, 10, 0, 0)

atmos = msise00.run(time, alt_km, glat, glon)

msplots.plotgtd(atmos)

show()



from dateutil.parser import parse
import numpy as np
from datetime import datetime, date
import typing

TIME_FMTS = typing.Union[str, datetime, np.datetime64]


def todt64(time: TIME_FMTS) -> np.ndarray:
    time = np.atleast_1d(time)

    if time.size == 1:
        time = np.atleast_1d(np.datetime64(time[0], dtype="datetime64[us]"))
    elif time.size == 2:
        time = np.arange(time[0], time[1], dtype="datetime64[h]")
    else:
        pass

    return time


def todatetime(time: TIME_FMTS) -> datetime:

    if isinstance(time, str):
        dtime = parse(time)
    elif isinstance(time, datetime):
        dtime = time
    elif isinstance(time, np.datetime64):
        dtime = time.astype(datetime)
    else:
        raise TypeError(f"{type(time)} not allowed")

    if not isinstance(dtime, datetime) and isinstance(dtime, date):
        dtime = datetime.combine(dtime, datetime.min.time())

    return dtime



"""
Call NRL MSISE-00 using f2py from Python
Michael Hirsch, Ph.D.

Original fortran code from
http://nssdcftp.gsfc.nasa.gov/models/atmospheric/msis/nrlmsise00/
"""
from datetime import datetime, date
import xarray
import numpy as np
import subprocess
import typing
from pathlib import Path
import io
import shutil

from .timeutils import todatetime
from .build import build

import geomagindices as gi

R = Path(__file__).resolve().parent
EXE = shutil.which("msise00_driver", path=str(R))
if not EXE:
    EXE = build()
if not EXE:
    raise RuntimeError("could not compile or find msise00_driver")

species = ["He", "O", "N2", "O2", "Ar", "Total", "H", "N", "AnomalousO"]
ttypes = ["Texo", "Tn"]
first = True


def run(
    time: datetime,
    altkm: float,
    glat: typing.Union[float, np.ndarray],
    glon: typing.Union[float, np.ndarray],
    indices: typing.Dict[str, typing.Any] = None,
) -> xarray.Dataset:
    """
    loops the rungtd1d function below. Figure it's easier to troubleshoot in Python than Fortran.
    """
    glat = np.atleast_2d(glat)
    glon = np.atleast_2d(glon)  # has to be here
    # %% altitude 1-D
    if glat.size == 1 and glon.size == 1 and isinstance(time, (str, date, datetime, np.datetime64)):
        atmos = rungtd1d(time, altkm, glat.squeeze()[()], glon.squeeze()[()], indices)
    # %% lat/lon grid at 1 altitude
    else:
        atmos = loopalt_gtd(time, glat, glon, altkm, indices)

    return atmos


def loopalt_gtd(
    time: datetime,
    glat: typing.Union[float, np.ndarray],
    glon: typing.Union[float, np.ndarray],
    altkm: typing.Union[float, typing.Sequence[float], np.ndarray],
    indices: typing.Dict[str, typing.Any] = None,
) -> xarray.Dataset:
    """
    loop over location and time

    time: datetime or numpy.datetime64 or list of datetime or np.ndarray of datetime
    glat: float or 2-D np.ndarray
    glon: float or 2-D np.ndarray
    altkm: float or list or 1-D np.ndarray
    """
    glat = np.atleast_2d(glat)
    glon = np.atleast_2d(glon)
    assert glat.ndim == glon.ndim == 2

    times = np.atleast_1d(time)
    assert times.ndim == 1

    atmos = xarray.Dataset()

    for t in times:
        print("computing", t)
        for i in range(glat.shape[0]):
            for j in range(glat.shape[1]):
                # atmos = xarray.concat((atmos, rungtd1d(t, altkm, glat[i,j], glon[i,j])),
                #                      data_vars='minimal',coords='minimal',dim='lon')
                atm = rungtd1d(t, altkm, glat[i, j], glon[i, j], indices)
                atmos = xarray.merge((atmos, atm))

    atmos.attrs = atm.attrs

    return atmos


def rungtd1d(
    time: datetime, altkm: np.ndarray, glat: float, glon: float, indices: typing.Dict[str, typing.Any] = None
) -> xarray.Dataset:
    """
    This is the "atomic" function looped by other functions
    """
    time = todatetime(time)
    # %% get solar parameters for date
    if not indices:
        indices = gi.getApF107(time, smoothdays=81).squeeze()
    # %% dimensions
    altkm = np.atleast_1d(altkm)
    if altkm.ndim != 1:
        raise ValueError("altitude read incorrectly")
    if not isinstance(glon, (int, float, np.int32, np.int64)):
        raise TypeError("single longitude only")
    if not isinstance(glat, (int, float, np.int32, np.int64)):
        raise TypeError("single latitude only")

    # %%
    iyd = time.strftime("%y%j")
    altkm = np.atleast_1d(altkm)
    # %%
    dens = np.empty((altkm.size, len(species)))
    temp = np.empty((altkm.size, len(ttypes)))
    for i, a in enumerate(altkm):
        cmd = [
            EXE,
            iyd,
            str(time.hour),
            str(time.minute),
            str(time.second),
            str(glat),
            str(glon),
            str(indices["f107s"]),
            str(indices["f107"]),
            str(indices["Ap"]),
            str(a),
        ]

        ret = subprocess.check_output(cmd, universal_newlines=True, stderr=subprocess.DEVNULL)
        f = io.StringIO(ret)
        dens[i, :] = np.genfromtxt(f, max_rows=1)
        temp[i, :] = np.genfromtxt(f, max_rows=1)

    dsf = {k: (("time", "alt_km", "lat", "lon"), v[None, :, None, None]) for (k, v) in zip(species, dens.T)}
    dsf.update(
        {
            "Tn": (("time", "alt_km", "lat", "lon"), temp[:, 1][None, :, None, None]),
            "Texo": (("time", "alt_km", "lat", "lon"), temp[:, 0][None, :, None, None]),
        }
    )

    atmos = xarray.Dataset(
        dsf,
        coords={"time": [time], "alt_km": altkm, "lat": [glat], "lon": [glon]},
        attrs={"species": species, "f107s": indices["f107s"], "f107": indices["f107"], "Ap": indices["Ap"]},
    )

    return atmos



#!/usr/bin/env python3
"""
python MSISE00.py -q -w tests/ref3.nc -a 200 -t 2017-03-01T12
"""
from pathlib import Path
import sys
import subprocess
import pytest
import xarray
import xarray.tests
import msise00
import msise00.worldgrid

R = Path(__file__).resolve().parent
altkm = 200.0
time = "2017-03-01T12"
script = R.parent / "MSISE00.py"


def test_one_alt_one_time():
    pytest.importorskip("netCDF4")
    ref = xarray.open_dataset(R / "ref3.nc")

    lat, lon = msise00.worldgrid.latlonworldgrid(30, 60)

    indices = {"f107": 79.3, "f107s": 76.6802469, "Ap": 39}
    try:
        dat_mod = msise00.run(time, altkm, lat, lon, indices).squeeze()
    except ConnectionError:
        pytest.skip("unable to download RecentIndices.txt")
    xarray.tests.assert_allclose(ref, dat_mod)


@pytest.mark.skipif(not script.is_file(), reason="demo script not available")
def test_script(tmp_path):
    pytest.importorskip("netCDF4")
    ref = xarray.open_dataset(R / "ref3.nc")

    fn = tmp_path / "test.nc"
    cmd = [sys.executable, str(script), "-q", "-w", str(fn), "-a", str(altkm), "-t", time, "-gs", "30", "60"]
    print(" ".join(cmd))
    subprocess.check_call(cmd)

    dat = xarray.open_dataset(fn)
    xarray.tests.assert_allclose(ref, dat)


if __name__ == "__main__":
    pytest.main(["-v", __file__])



#!/usr/bin/env python
"""
Example:

lat / lon grid at altitude example
"""
from datetime import datetime
from matplotlib.pyplot import show

import msise00.plots as msplots
import msise00
from msise00.worldgrid import latlonworldgrid

glat, glon = latlonworldgrid(10.0, 20.0)
alt_km = 200.0
time = datetime(2015, 12, 13, 10, 0, 0)

atmos = msise00.run(time, alt_km, glat, glon)

msplots.plotgtd(atmos)

show()



import numpy as np
from typing import Tuple, Union


def latlonworldgrid(latstep: Union[float, int] = 5, lonstep: Union[float, int] = 5) -> Tuple[np.ndarray, np.ndarray]:
    lat = np.arange(-90.0, 90 + latstep, latstep)
    lon = np.arange(-180.0, 180 + lonstep, lonstep)
    glon, glat = np.meshgrid(lon, lat)

    return glat, glon



#!/usr/bin/env python
"""
understanding sun apparent position over Earth in geodetic lat,lon
"""
from astropy.time import Time
from astropy.coordinates import get_sun, AltAz, EarthLocation
import astropy.units as u
import numpy as np
from matplotlib.pyplot import figure, show
from pymap3d import aer2geodetic

obslla = (0, 0, 0)

midnight = Time("2015-01-01T00:00")
delta_midnight = np.linspace(0, 365, 1000) * u.day
time = midnight + delta_midnight

obs = EarthLocation(lat=obslla[0], lon=obslla[1], height=obslla[2])
sun = get_sun(time=time)
aaf = AltAz(obstime=time, location=obs)
sloc = sun.transform_to(aaf)
# %%
time = time.to_datetime()

fg = figure()
ax = fg.subplots(2, 1, sharex=True)
ax[0].plot(time, sloc.alt)
ax[0].set_title("sun elevation")
ax[0].set_ylabel("elevation [deg]")

ax[1].plot(time, sloc.az)
ax[1].set_title("sun azimuth")
ax[1].set_ylabel("azimuth [deg]")
ax[1].set_xlabel("time")

fg.suptitle(f"sun over 1 year @ lat,lon,alt: {obslla}")

# %%

lat, lon, alt = aer2geodetic(sloc.az.value, sloc.alt.value, sloc.distance.value, *obslla)

ax = figure().gca()
ax.plot(time, lat)
ax.set_title("subsolar latitude vs. time")
ax.set_ylabel("latitude [deg]")
ax.set_xlabel("time")

show()



from pathlib import Path
import xarray
from datetime import datetime
from matplotlib.pyplot import figure, close
from matplotlib.ticker import ScalarFormatter

try:
    from pymap3d import aer2geodetic
except ImportError:
    aer2geodetic = None
try:
    from astropy.time import Time
    from astropy.coordinates import get_sun, EarthLocation, AltAz
except ImportError:
    Time = None

sfmt = ScalarFormatter(useMathText=True)  # for 10^3 instead of 1e3
sfmt.set_powerlimits((-2, 2))
sfmt.set_scientific(True)
sfmt.set_useOffset(False)


def plotgtd(atmos: xarray.Dataset, rodir: Path = None):
    #
    if rodir:
        rodir = Path(rodir).expanduser()

    atmos = atmos.squeeze()
    tmp = atmos["N2"]

    if tmp.ndim == 1:
        if "alt_km" in tmp.dims:
            plot1dalt(atmos)
        elif "time" in tmp.dims:
            plot1dtime(atmos)
        else:
            raise NotImplementedError("didnt handle this plotting case yet. Should be straightforward.")
    elif tmp.ndim == 2:
        if "lat" in tmp.dims and "lon" in tmp.dims:
            plot2dlatlon(atmos, rodir)
        elif "time" in tmp.dims:
            for t in atmos.time:
                plot1dalt(atmos.sel(time=t), rodir)
        else:
            raise NotImplementedError("didnt handle this plotting case yet. Should be straightforward.")
    elif tmp.ndim in (3, 4):  # lat/lon grid vs. time
        plot4d(atmos, rodir)
    else:  # single point
        print(atmos)


def plot4d(atmos: xarray.Dataset, rodir: Path = None):

    for t in atmos.time:
        if Time is not None and aer2geodetic is not None:
            time = Time(str(t.values))
            obs = EarthLocation(0, 0)  # geodetic lat,lon = 0,0 arbitrary
            sun = get_sun(time=time)
            aaf = AltAz(obstime=time, location=obs)
            sloc = sun.transform_to(aaf)
            slat, slon = aer2geodetic(sloc.az.value, sloc.alt.value, sloc.distance.value, 0, 0, 0)[:2]
            plot2dlatlon(atmos.sel(time=t), rodir, slat, slon)
        else:
            plot2dlatlon(atmos.sel(time=t), rodir)


def plot2dlatlon(atmos: xarray.Dataset, rodir: Path = None, slat: float = None, slon: float = None):

    fg = figure(figsize=(8, 8))
    ax = fg.subplots(4, 2, sharex=True).ravel()

    fg.suptitle(str(atmos.time.values.squeeze())[:-13] + f" alt.(km) {atmos.alt_km.item()}\n" f"Ap={atmos.Ap}  F10.7={atmos.f107}")

    j = 0

    for s in atmos.species:
        if s == "Total":
            continue

        a = ax[j]

        hi = a.imshow(
            atmos[s].squeeze(),
            aspect="auto",
            interpolation="none",
            cmap="viridis",
            extent=(atmos.lon[0], atmos.lon[-1], atmos.lat[0], atmos.lat[-1]),
        )

        fg.colorbar(hi, ax=a, format=sfmt)
        # %% sun icon moving
        if slat is not None and slon is not None:
            a.plot(slon, slat, linestyle="none", marker="o", markersize=5, color="w")

        a.set_title(f"Density: {s}")
        a.set_xlim(-180, 180)
        a.set_ylim(-90, 90)
        a.autoscale(False)
        j += 1

    for k in range(0, 6 + 2, 2):
        ax[k].set_ylabel("latitude (deg)")
    for k in (6, 7):
        ax[k].set_xlabel("longitude (deg)")

    if rodir:
        ofn = rodir / (f"{atmos.alt_km.item():.1f}_" + str(atmos.time.values.squeeze())[:-13] + ".png")
        writeplot(fg, ofn)


def plot1dalt(atmos: xarray.Dataset, odir: Path = None):

    footer = f"\n({atmos.lat.item()},{atmos.lon.item()})  Ap {atmos.Ap}  F10.7 {atmos.f107}"

    z = atmos.alt_km.values

    # %% number density
    fg = figure()
    ax = fg.gca()
    for s in atmos.species:
        if s == "Total":
            continue
        ax.semilogx(atmos[s].squeeze(), z, label=s)
    ax.legend(loc="best")
    ax.set_xlim(left=1e3)
    ax.set_ylabel("altitude [km]")
    ax.set_xlabel("density [m$^{-3}$]")
    ax.grid(True)
    ax.set_title("Number Density from MSISE-00" + footer)
    if odir:
        ofn = odir / (f"number-density_" + str(atmos.time.values.squeeze())[:-13] + ".png")
        writeplot(fg, ofn)

    # %% total mass
    fg = figure()
    ax = fg.gca()
    ax.semilogx(atmos["Total"].squeeze(), z)
    ax.set_xlabel("Total Mass Density [kg m$^{-3}$]")
    ax.set_ylabel("altitude [km]")
    ax.grid(True)
    ax.set_title("Total Mass Density from MSISE-00" + footer)

    if odir:
        ofn = odir / (f"total-density_" + str(atmos.time.values.squeeze())[:-13] + ".png")
        writeplot(fg, ofn)

    # %% temperature
    fg = figure()
    ax = fg.gca()
    ax.plot(atmos["Tn"].squeeze(), z)
    ax.set_xlabel("Temperature [K]")
    ax.set_ylabel("altitude [km]")
    ax.grid(True)
    ax.set_title("Temperature from MSISE-00" + footer)

    if odir:
        ofn = odir / (f"temperature_" + str(atmos.time.values.squeeze())[:-13] + ".png")
        writeplot(fg, ofn)


def plot1dtime(atmos: xarray.Dataset, odir: Path = None):

    footer = f"\n({atmos.lat.item()},{atmos.lon.item()})  alt: {atmos.alt_km.item()} km,  Ap {atmos.Ap}  F10.7 {atmos.f107}"

    t = atmos.time.values.astype("datetime64[us]").astype(datetime)

    # %% number density
    fg = figure()
    ax = fg.gca()
    for s in atmos.species:
        if s == "Total":
            continue
        ax.plot(t, atmos[s].squeeze(), label=s)
    ax.legend(loc="best")
    ax.set_ylabel("density [m$^{-3}$]")
    ax.set_xlabel("time [UTC]")
    ax.grid(True)
    ax.set_title("Number Density from MSISE-00" + footer)
    if odir:
        ofn = odir / (f"number-density_" + str(atmos.time.values.squeeze())[:-13] + ".png")
        writeplot(fg, ofn)

    # %% total mass
    fg = figure()
    ax = fg.gca()
    ax.plot(t, atmos["Total"].squeeze())
    ax.set_ylabel("Total Mass Density [kg m$^{-3}$]")
    ax.set_xlabel("time [UTC]")
    ax.grid(True)
    ax.set_title("Total Mass Density from MSISE-00" + footer)

    if odir:
        ofn = odir / (f"total-density_" + str(atmos.time.values.squeeze())[:-13] + ".png")
        writeplot(fg, ofn)

    # %% temperature
    fg = figure()
    ax = fg.gca()
    ax.plot(t, atmos["Tn"].squeeze())
    ax.set_ylabel("Temperature [K]")
    ax.set_xlabel("time [UTC]")
    ax.grid(True)
    ax.set_title("Temperature from MSISE-00" + footer)

    if odir:
        ofn = odir / (f"temperature_" + str(atmos.time.values.squeeze())[:-13] + ".png")
        writeplot(fg, ofn)


def writeplot(fg, ofn: Path):
    print("writing", ofn)
    fg.savefig(ofn, dpi=100, bbox_inches="tight")
    close(fg)



#!/usr/bin/env python
"""
Run MSISE00, save to NetCDF4 (HDF5) and/or plot.

Example:
Poker Flat Research Range altitude profile:

MSISE00.py -t 2018-01-01 -c 65 -148 -a 0.2

"""
from pathlib import Path
import numpy as np
import msise00
from argparse import ArgumentParser
from msise00.worldgrid import latlonworldgrid

try:
    from matplotlib.pyplot import show
    import msise00.plots as msplots
except ImportError as e:
    print(e)
    msplots = None  # type: ignore
try:
    import seaborn as sns

    sns.set_style("ticks")
except ImportError:
    pass


def main():
    p = ArgumentParser(description="calls MSISE-00 from Python, save to NetCDF4 and/or plot")
    p.add_argument("-t", "--time", help="time or times", nargs="+", required=True)
    p.add_argument(
        "-a", "--altkm", help="altitude (km). scalar, or (start,stop,step) or list of alts.", type=float, nargs="+", required=True
    )
    p.add_argument("-c", "--latlon", help="geodetic latitude/longitude (deg)", metavar=("lat", "lon"), type=float, nargs=2)
    p.add_argument("-o", "--odir", help="directory to write plots to")
    p.add_argument("-w", help="NetCDF4 .nc filename to write")
    p.add_argument("-gs", help="geographic grid spacing (lat, lon)", nargs=2, type=float, default=(10, 10))
    p.add_argument("-q", "--quiet", help="disable plotting", action="store_true")
    P = p.parse_args()

    # %% altitude
    if len(P.altkm) == 1:
        altkm = P.altkm[0]
    elif len(P.altkm) == 3:
        altkm = np.arange(*P.altkm)
    else:
        altkm = P.altkm
    # %% latlon
    if P.latlon is not None:
        glat, glon = P.latlon
    else:
        glat, glon = latlonworldgrid(*P.gs)
    # %% run
    atmos = msise00.run(P.time, altkm, glat, glon)
    # %% save
    if P.w:
        ncfn = Path(P.w).expanduser()
        print("saving", ncfn)
        # NOTE: .squeeze() avoids ValueError: unsupported dtype for netCDF4 variable: datetime64[ns]
        atmos.squeeze().to_netcdf(ncfn)
    # %% plot
    if msplots is not None and not P.quiet:
        msplots.plotgtd(atmos, P.odir)
        show()
    else:
        print("skipped plots")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
r"""
On Windows with Octave >= 5, you may need to add to PATH like:
    C:\Octave\Octave-5.1.0.0\mingw64\bin
to get at octave.exe instead of octave.vbs
"""
from pathlib import Path
import subprocess
import pytest
import shutil

R = Path(__file__).parent
Rcode = R.parent / "matlab"

OCTAVE = shutil.which("octave-cli")
MATLAB = shutil.which("matlab")


@pytest.mark.skipif(not MATLAB, reason="Matlab not available")
def test_matlab_api():
    subprocess.check_call([MATLAB, "-batch", "runtests"], cwd=R, timeout=60)


@pytest.mark.skipif(not OCTAVE, reason="octave not found")
def test_octave_api():
    subprocess.check_call([OCTAVE, "test_mod.m"], cwd=R, timeout=60)


if __name__ == "__main__":
    pytest.main(["-r", "a", "-v", __file__])
